#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "main.h"
#include "Graphics.h"
#include "System.h"
#include "maze.h"

#define SCREEN_WIDTH    (MAZE_WIDTH  * TILE_SIZE + WALL_WIDTH)
#define SCREEN_HEIGHT   (MAZE_HEIGHT * TILE_SIZE + WALL_WIDTH)

CMaze maze;

bool checkWall(int x, int y, bool& isHorizontal, Color& color)
{
    Color   c1, c2;
    bool    inter1, inter2;

    inter1 = maze.checkIntersection(x, y, c1);

    if (isHorizontal == true)
        inter2 = maze.checkIntersection(x + 1, y, c2);
    else
        inter2 = maze.checkIntersection(x, y + 1, c2);

    if (inter1 == true && inter2 == false)
    {
        if (isHorizontal == true)
            maze.getTile(x, y)->topVisited = true;
        else
            maze.getTile(x, y)->leftVisited = true;

        color = c1;
        return false;
    }
    else if (inter1 == false && inter2 == true)
    {
        if (isHorizontal == true)
            maze.getTile(x, y)->topVisited = true;
        else
            maze.getTile(x, y)->leftVisited = true;

        color = c2;
        return false;
    }
    else if (inter1 == true && inter2 == true)
    {
        if (isHorizontal == true)
            maze.getTile(x, y)->topVisited = true;
        else
            maze.getTile(x, y)->leftVisited = true;
    }

    return true;
}

bool checkFinished()
{
    int nbVisits = 0;

    for (int x = 0; x < MAZE_WIDTH; x++)
        for (int y = 0; y < MAZE_HEIGHT; y++)
        {
            if (maze.getTile(x, y)->topVisited == true)
                nbVisits++;

            if (maze.getTile(x, y)->leftVisited == true)
                nbVisits++;
        }

    return (nbVisits == (MAZE_WIDTH - 1) * (MAZE_HEIGHT - 2) + (MAZE_WIDTH - 2) * (MAZE_HEIGHT - 1));
}

// choose a random wall inside maze
void chooseWall()
{
    int     x, y;
    Color   c;
    bool    isHorizontal;

    while(true)
    {
        if (checkFinished() == true)
            break;

        isHorizontal = (rand() & 1 ? false : true);

        if (isHorizontal == true)
        {
            x = rand() % MAZE_WIDTH;
            y = 1 + (rand() % (MAZE_HEIGHT - 1));

            if (x == 0)
            {
                if (maze.checkIntersection(1, y, c) == false)
                {
                    maze.getTile(x, y)->topIsActive = true;
                    maze.getTile(x, y)->topColor = Color(32 + rand() % 224,
                                                         32 + rand() % 224,
                                                         32 + rand() % 224);
                    break;
                }
            }
            else if (x == MAZE_WIDTH - 1)
            {
                if (maze.checkIntersection(x, y, c) == false)
                {
                    maze.getTile(x, y)->topIsActive = true;
                    maze.getTile(x, y)->topColor = Color(32 + rand() % 224,
                                                         32 + rand() % 224,
                                                         32 + rand() % 224);
                    break;
                }
            }
            else if (checkWall(x, y, isHorizontal, c) == false)
            {
                maze.getTile(x, y)->topIsActive = true;
                maze.getTile(x, y)->topColor = c;
                break;
            }
        }
        else
        {
            x = 1 + (rand() % (MAZE_WIDTH - 1));
            y = rand() % MAZE_HEIGHT;

            if (y == 0)
            {
                if (maze.checkIntersection(x, 1, c) == false)
                {
                    maze.getTile(x, y)->leftIsActive = true;
                    maze.getTile(x, y)->leftColor = Color(32 + rand() % 224,
                                                          32 + rand() % 224,
                                                          32 + rand() % 224);
                    break;
                }
            }
            else if (y == MAZE_HEIGHT - 1)
            {
                if (maze.checkIntersection(x, y, c) == false)
                {
                    maze.getTile(x, y)->leftIsActive = true;
                    maze.getTile(x, y)->leftColor = Color(32 + rand() % 224,
                                                          32 + rand() % 224,
                                                          32 + rand() % 224);
                    break;
                }
            }
            else if (checkWall(x, y, isHorizontal, c) == false)
            {
                maze.getTile(x, y)->leftIsActive = true;
                maze.getTile(x, y)->leftColor = c;
                break;
            }
        }
    }
}

int main(int argc, char* argv[])
{
    // init the window
    gfx.init("Tree Maze 10", SCREEN_WIDTH, SCREEN_HEIGHT);
    gfx.init2D();

    // main loop
    while (sys.isQuitRequested() == false)
    {
        chooseWall();
        gfx.clearScreen(Color(0, 0, 0));
        maze.draw();

        sys.processEvents();
        gfx.render();
        sys.wait(50);
    }

    gfx.quit();

    return EXIT_SUCCESS;
}
