#include "maze.h"
#include <stdlib.h>
#include <time.h>

void CMaze::drawWall(int x, int y, bool isHorizontal, Color c)
{
    int sx = x * TILE_SIZE;
    int sy = y * TILE_SIZE;

    if (isHorizontal == true)
    {
        int ex = sx + TILE_SIZE - 1;
        int ey = sy + WALL_WIDTH - 1;

        for (int yy = sy; yy <= ey; yy++)
            for (int xx = sx; xx <= ex; xx++)
                gfx.setPixel(xx, yy, c);
    }
    else
    {
        int ex = sx + WALL_WIDTH - 1;
        int ey = sy + TILE_SIZE + WALL_WIDTH - 1;

        for (int yy = sy; yy <= ey; yy++)
            for (int xx = sx; xx <= ex; xx++)
                gfx.setPixel(xx, yy, c);
    }
}

// choose a random place to put a trunk
STile* CMaze::chooseTrunk(bool& isHorizontal)
{
    int x, y;
    Color   c;

    while (true)
    {
        int dir = rand() % 4;

        if (dir == 0)
        {
            isHorizontal = true;
            x = 0;
            y = 1 + (rand() % (MAZE_HEIGHT - 1));

            if (checkIntersection(x + 1, y, c) == false)
                break;
        }
        else if (dir == 1)
        {
            isHorizontal = true;
            x = MAZE_WIDTH - 1;
            y = 1 + (rand() % (MAZE_HEIGHT - 1));

            if (checkIntersection(x, y, c) == false)
                break;
        }
        else if (dir == 2)
        {
            isHorizontal = false;
            x = 1 + (rand() % (MAZE_WIDTH - 1));
            y = 0;

            if (checkIntersection(x, y + 1, c) == false)
                break;
        }
        else
        {
            isHorizontal = false;
            x = 1 + (rand() % (MAZE_WIDTH - 1));
            y = MAZE_HEIGHT - 1;

            if (checkIntersection(x, y, c) == false)
                break;
        }
    }

    return getTile(x, y);
}

// choose a random place to put an island
STile* CMaze::chooseIsland(bool& isHorizontal)
{
    int x, y;
    Color   c;

    while (true)
    {
        isHorizontal = (rand() & 1 ? false : true);

        if (isHorizontal == true)
        {
            x = 1 + (rand() % (MAZE_WIDTH - 2));
            y = 1 + (rand() % (MAZE_HEIGHT - 1));

            if (checkIntersection(x, y, c) == false &&
                checkIntersection(x + 1, y, c) == false)
                break;
        }
        else
        {
            x = 1 + (rand() % (MAZE_WIDTH - 1));
            y = 1 + (rand() % (MAZE_HEIGHT - 2));

            if (checkIntersection(x, y, c) == false &&
                checkIntersection(x, y + 1, c) == false)
                break;
        }
    }

    return getTile(x, y);
}

CMaze::CMaze()
{
    srand(time(NULL));

    // init the maze
    for (int y = 0; y <= MAZE_HEIGHT; y++)
        for (int x = 0; x <= MAZE_WIDTH; x++)
        {
            tiles[x][y].topIsActive  = false;
            tiles[x][y].leftIsActive = false;
            tiles[x][y].topVisited  = false;
            tiles[x][y].leftVisited = false;
        }

    // init left and right borders
    for (int y = 0; y < MAZE_HEIGHT; y++)
    {
        tiles[0][y].leftIsActive = true;
        tiles[0][y].leftColor = Color(255, 255, 255);

        tiles[MAZE_WIDTH][y].leftIsActive = true;
        tiles[MAZE_WIDTH][y].leftColor = Color(255, 255, 255);
    }

    // init top and bottom borders
    for (int x = 0; x < MAZE_WIDTH; x++)
    {
        tiles[x][0].topIsActive = true;
        tiles[x][0].topColor = Color(255, 255, 255);

        tiles[x][MAZE_HEIGHT].topColor = Color(255, 255, 255);
        tiles[x][MAZE_HEIGHT].topIsActive = true;
    }

    // init the islands
    for (int i = 0; i < NB_ISLANDS; ++i)
    {
        bool isHorizontal;
        Color   color(32 + rand() % 224,
                      32 + rand() % 224,
                      32 + rand() % 224);

        STile*  tile = chooseIsland(isHorizontal);

        if (isHorizontal == true)
        {
            tile->topIsActive = true;
            tile->topColor = color;
        }
        else
        {
            tile->leftIsActive = true;
            tile->leftColor = color;
        }
    }
}

void CMaze::draw()
{
    // draw the maze
    for (int y = 0; y <= MAZE_HEIGHT; y++)
        for (int x = 0; x <= MAZE_WIDTH; x++)
        {
            if (tiles[x][y].leftIsActive == true)
                drawWall(x, y, false, tiles[x][y].leftColor);

            if (tiles[x][y].topIsActive == true)
                drawWall(x, y, true, tiles[x][y].topColor);
        }
}

bool CMaze::hasWallUp(int x, int y, Color& c)
{
    c = getTile(x, y - 1)->leftColor;
    return getTile(x, y - 1)->leftIsActive;
}

bool CMaze::hasWallDown(int x, int y, Color& c)
{
    c = getTile(x, y)->leftColor;
    return getTile(x, y)->leftIsActive;
}

bool CMaze::hasWallLeft(int x, int y, Color& c)
{
    c = getTile(x - 1, y)->topColor;
    return getTile(x - 1, y)->topIsActive;
}

bool CMaze::hasWallRight(int x, int y, Color& c)
{
    c = getTile(x, y)->topColor;
    return getTile(x, y)->topIsActive;
}

bool CMaze::checkIntersection(int x, int y, Color& c)
{
    if (hasWallUp(x, y, c) == true)
        return true;

    if (hasWallDown(x, y, c) == true)
        return true;

    if (hasWallLeft(x, y, c) == true)
        return true;

    if (hasWallRight(x, y, c) == true)
        return true;

    return false;
}
